/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: ioport_itifg.c,v 7.0.2.1 2000/11/29 14:58:30 cvs Exp $
 *
 */

/* Feature test switches */

/* System headers */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <termios.h>

#include <sys/ioctl.h>
#include <sys/param.h>

#ifndef __NetBSD__
#include <getopt.h>
#endif

#ifdef linux
#  include <linux/limits.h>	/* PATH_MAX */
#endif

/* Local headers */

#include "itifgExt.h"

/* Macros */

/* intervals smaller than 20ms are not useful (sched) */
#define MIN_SAMPLING_TIME	2000
/* this is the build-in timeout for read the io-ports */
#define MAX_SAMPLING_TIME	ITI_WAIT_IOP * 10
/* 1s (in ms) is a good value */
#define DEF_SAMPLING_TIME	1000	

/* File scope variables */

static int device_desc = -1;

/* External variables */

extern char *optarg;
extern int optind, opterr, optopt;

/* External functions */

/* Structures and unions */

/* Signal catching functions */

void print_stats (int);

void
print_stats (int signum)
{
  struct iti_acc_t accounting_info;
  
  if (ioctl (device_desc, GIOC_GE_GET_STATS, &accounting_info) < 0)
    {
      perror ("GIOC_GE_GET_STATS");
      exit (EXIT_FAILURE);
    }
  
  printf ("statistics:\n");
  printf ("events captured %ld,\n", accounting_info.events_captured);
  printf ("bytes valid %ld,\n", accounting_info.bytes_valid);
  printf ("bytes lost %ld,\n", accounting_info.bytes_lost);
  
  exit (EXIT_SUCCESS);
}

/* Functions */

/* Main */

int
main (int argc, char *argv[])
{
  int option, flags = 0, board = 0;
  int outport_value = -1;
  int sampling_time = DEF_SAMPLING_TIME;

  u_char sampling_value;
  struct timeval sampling_time_str;
  struct timeval sampling_timestamp;
  struct timezone sampling_timezone;
  time_t *sampling_timestamp_ptr = (time_t*)&sampling_timestamp.tv_sec;

  char device_name[PATH_MAX];

  while (TRUE)
    {
      if ((option = getopt (argc, argv, "b:t:w:?")) == -1)
	break;
      switch (option)
	{
	case 'b':
	  if (!optarg)
	    printf ("No board selected, using default (%d).\n", board);
	  else
	    {
	      board = atoi (optarg);
	      if (board < 0 || board >= ITI_BOARDS_MAX)
		{
		  printf ("Wrong board selected (%d).\n", board);
		  exit (EXIT_FAILURE);
		}
	    }
	  break;
	case 't':
	  if (!optarg)
	    printf ("No sampling time given, using default (%dms).\n",
		    sampling_time);
	  else
	    {
	      sampling_time = atoi (optarg);
	      if (sampling_time &&
		  (sampling_time < MIN_SAMPLING_TIME ||
		   sampling_time > MIN_SAMPLING_TIME))
		{
		  printf ("Wrong sampling interval given (%dms).\n",
			  sampling_time);
		  exit (EXIT_FAILURE);
		}
	    }
	  break;
	case 'w':
	  if (!optarg)
	    printf ("No outport value selected, using default (0).\n");
	  else
	    {
	      outport_value = atoi (optarg);
	      if (outport_value < 0 || outport_value > 255)
		{
		  printf ("Wrong outport value selected (%d).\n", board);
		  exit (EXIT_FAILURE);
		}
	    }
	  break;
	case '?':
	default:
	  printf ("Usage: ioport_itifg -b board (0) [0-%d]\n",
		  ITI_BOARDS_MAX - 1);
	  printf ("                    -t sampling time (%dms) [%d-%dms]\n",
		  DEF_SAMPLING_TIME, MIN_SAMPLING_TIME, MAX_SAMPLING_TIME);
	  printf ("                    -w outport value (0) [0-255]\n");
	  exit (EXIT_FAILURE);
	}
    }
  
  flags = O_RDWR;
  if (sampling_time)
    /* do not wait for data strobe */
    flags |= O_NONBLOCK;
  
  snprintf (device_name, PATH_MAX, "%s%d%s", ITI_NAME_PREFIX, board,
	    ITI_NAME_PORT_POSTFIX);
  
  if ((device_desc = open (device_name, flags)) < 0)
    {
      perror ("open");
      exit (EXIT_FAILURE);
    }
  
  if (signal (SIGINT, print_stats) < 0)
    {
      perror ("signal");
      exit (EXIT_FAILURE);
    }

  if (sampling_time)
    {
      sampling_time -= 10; /* scheduling interval */
      sampling_time_str.tv_sec = sampling_time / 1000;
      sampling_time_str.tv_usec = (sampling_time % 1000) * 1000;
    }
  
  if (outport_value >= 0)
    {
      if ((write (device_desc, &outport_value, 1) != 1))
	{
	  perror ("write");
	  exit (EXIT_FAILURE);
	}
      printf ("Done.\n");
      exit (EXIT_SUCCESS);
    }
  
  /* you can stop this only with Ctrl+C */
  for (;;)
    {
      if (gettimeofday (&sampling_timestamp, &sampling_timezone) < 0)
	{
	  perror ("gettimeofday");
	  exit (EXIT_FAILURE);
	}
      
      while (TRUE)
	{
	  if ((read (device_desc, &sampling_value, 1) != 1))
	    {
	      if (!sampling_time && errno == ENODATA)
		printf ("timeout reading the io-port.\n"); 
	      else
		{
		  perror ("read");
		  exit (EXIT_FAILURE);
		}
	    }
	  else
	    break;
	}

      printf ("%s %3ldms: 0x%X\n",
	      strtok (ctime (sampling_timestamp_ptr), "\n"),
	      (long)sampling_timestamp.tv_usec / 1000, sampling_value);
     
      if (sampling_time)
	usleep (sampling_time_str.tv_sec * 1000000 +
		sampling_time_str.tv_usec);
    }

  if (close (device_desc) < 0)
    {
      perror ("close");
      exit (EXIT_FAILURE);
    }

  exit (EXIT_SUCCESS);
}

